package chat.client;

import chat.remote.RemoteConnection;
import chat.remote.RemoteServer;
import chat.remote.RemoteClient;

/**
 * A text based RMI chat client.
 */
public class ClientImpl extends java.rmi.server.UnicastRemoteObject implements RemoteClient {

  private RemoteServer chatServer;
  private RemoteConnection chatConnection;
  private String clientName;


  //
  // -- CONSTRUCTORS -------------------------------------------------
  //

  public ClientImpl(String serverURL, String clientName) throws java.rmi.RemoteException {
    try {
      chatServer = (RemoteServer) java.rmi.Naming.lookup(serverURL);
    } catch (java.net.MalformedURLException e) {
      throw new java.rmi.RemoteException("The URL of the server if not well formed");
    } catch (java.rmi.NotBoundException e) {
      throw new java.rmi.RemoteException("The server was not found at the given URL");
    }
    this.clientName = clientName;
    chatConnection = chatServer.logon(clientName, this);
    if (chatConnection == null) throw new java.rmi.RemoteException("Cannot used the given name");
  }


  //
  // -- PUBLIC METHODS -------------------------------------------------
  //
  
  public static void main(String[] args) throws Exception  {
    if (args.length < 2) {
      System.out.println("You must give the URL of the chat server and the name of the client as arguments");
      System.out.println(" ex : ClientImpl rmi://clio.unice.fr/Server bob");
      System.exit(1);
    }
    // Create and install a security manager
    if (System.getSecurityManager() == null) {
      System.setSecurityManager(new java.rmi.RMISecurityManager());
    }
    ClientImpl client = new ClientImpl(args[0], args[1]);
    client.startClient();
  }
  

  public void startClient() throws java.io.IOException {
    java.io.BufferedReader in = new java.io.BufferedReader(new java.io.InputStreamReader(System.in));
    while (true) {
      String msg = in.readLine();
      if (msg == null || msg.length() == 0) break;
      // ***********************
      // Utiliser l'object ChatConnection pour envoyer le msg au serveur
      // ***********************
    }
    // ***********************
    // Utiliser l'object ChatConnection pour se deconnecter du serveur
    // ***********************
    System.exit(0);
  }


  //
  // -- implements RemoteClient -------------------------------------------------
  //

  public void speak(String who, String msg) {
    System.out.println(who + ": " + msg);
  }

}
