package chat.server;

import chat.remote.RemoteClient;


public interface Server {

  /**
   * Adds a new client of given name. The existing client are notified of the addition.
   * @param clientName the name of the new client
   * @param client the client object
   * @return true if the client was added or false if a client of same name already existed
   */
  public boolean addClient(String clientName, RemoteClient client);


  /**
   * Removes an existing client of given name.
   * @param clientName the name of the client to remove
   * @return true if the client was removed or false if it didn't exist
   */
  public boolean removeClient(String clientName);


  /**
   * Returns the names of all existing client.
   * @param clientName the name of the client to get
   * @return the client object matching the name or null if not found
   */
  public String[] getClientsName();


  /**
   * Returns an existing client of given name.
   * @param clientName the name of the client to get
   * @return the client object matching the name or null if not found
   */
  public RemoteClient getClient(String clientName);


  /**
   * Broadcasts a message to all clients.
   * @param from who is sending
   * @param msg the message to broadcast
   */
  public void broadcastMessage(String from, String msg);


  /**
   * Targets a message to one clients.
   * @param msg the message to broadcast
   * @param from who is sending
   * @param to who is receiving the message
   * @return whether the destination client (named <code>to</code>) 
   * existed and was able to received the message
   */
  public boolean targetMessage(String msg, String from, String to);


}
