package chat.server;

import chat.remote.RemoteClient;

public class ServerImpl implements Server {

  //
  // -- PRIVATE MEMBERS -------------------------------------------------
  //

  private java.util.HashMap clients;


  //
  // -- CONSTRUCTORS -------------------------------------------------
  //

  public ServerImpl() {
	clients = new java.util.HashMap();
  }


  //
  // -- PUBLIC METHODS -------------------------------------------------
  //

  //
  // -- implements Server -------------------------------------------------
  //

  public boolean addClient(String clientName, RemoteClient client) {
	synchronized(clients) {
	  if (clients.get(clientName) != null) return false;
	  clients.put(clientName, client);
	}
	System.out.println(clientName + " logged on");
	notifyAllClients();
	return true;
  }


  public boolean removeClient(String clientName) {
	boolean result = removeClientInternal(clientName);
	if (result) {
	  notifyAllClients();
	  System.out.println(clientName + " logged off");
	}
	return result;
  }


  public String[] getClientsName() {
	synchronized(clients) {
	  int n = clients.size();
	  String[] result = new String[n];
	  java.util.Iterator it = clients.keySet().iterator();
	  int i = 0;
	  while (it.hasNext()) {
		result[i++] = (String) it.next();
	  }
	  return result;
	}
  }


  public RemoteClient getClient(String clientName) {
	synchronized(clients) {
	  return (RemoteClient) clients.get(clientName);
	}
  }


  public void broadcastMessage(String from, String msg) {
	new RunnableClientCaller(entriesIterator(), new ClientSpeakCall(from, msg));
  }


  public boolean targetMessage(String msg, String from, String to) {
	RemoteClient clientReceiver = getClient(to);
	RemoteClient clientSender = getClient(from);
	if (clientReceiver == null) return false;
	msg = "[ " + msg + " ]";
	try {
	  clientReceiver.speak(from, msg);
	} catch (java.rmi.RemoteException err) {
	  System.out.println("Dropping " + to + " due to remote error");
	  removeClient(to);
	  return false;
	}
	try {
	  clientSender.speak(from, msg);
	} catch (java.rmi.RemoteException err) {
	  System.out.println("Dropping " + from + " due to remote error");
	  removeClient(from);
	  return true;
	}
	return true;
  }




  //
  // -- PRIVATE METHODS -------------------------------------------------
  //

  private boolean removeClientInternal(String clientName) {
	synchronized(clients) {
	  return clients.remove(clientName) != null;
	}
  }


  private java.util.Iterator entriesIterator() {
	java.util.HashMap cloneClients = cloneClients();
	return cloneClients.entrySet().iterator();
  }

  private java.util.HashMap cloneClients() {
	synchronized (clients) {
	  return (java.util.HashMap) clients.clone();
	}
  }

  private void notifyAllClients() {
	new RunnableClientCaller(entriesIterator(), new ClientChangeNotificationCall());
  }


  //
  // -- INNER CLASSES -------------------------------------------------
  //

  private interface ClientCall {
	public void doCall(RemoteClient client, String name) throws java.rmi.RemoteException;
  }



  private class ClientChangeNotificationCall implements ClientCall {
	public void doCall(RemoteClient client, String name) throws java.rmi.RemoteException {
	  //client.clientChangeNotification();
	}
  }



  private class ClientSpeakCall implements ClientCall {
	private String from;
	private String what;

	public ClientSpeakCall(String from, String what) {
	  this.from = from;
	  this.what = what;
	}

	public void doCall(RemoteClient client, String name) throws java.rmi.RemoteException {
	  client.speak(from, what);
	}
  }



  private class RunnableClientCaller implements Runnable {

	private java.util.Iterator iterator;
	private ClientCall clientCall;

	public RunnableClientCaller(java.util.Iterator iterator, ClientCall clientCall) {
	  this.iterator = iterator;
	  this.clientCall = clientCall;
	  Thread t = new Thread(this);
	  t.start();
	}

	public void run() {
	  while (iterator.hasNext()) {
		java.util.Map.Entry entry = (java.util.Map.Entry) iterator.next();
		String clientName = (String) entry.getKey();
		RemoteClient client = (RemoteClient) entry.getValue();
		try {
		  clientCall.doCall(client, clientName);
		} catch (java.rmi.RemoteException e) {
		  removeClient(clientName);
		}
	  }
	}
  } // end inner class RunnableClientNotifier

}
